import os
import sys
import json
import requests
from solders.pubkey import Pubkey
from solana.rpc.api import Client,RPCException
from spl.token.client import Token
from solders.keypair import Keypair
from construct import Struct as cStruct
from solana.transaction import AccountMeta,Transaction
from solders.instruction import Instruction
from spl.token.instructions import close_account, CloseAccountParams
from abstract_security import get_env_value
from solana.rpc.types import TokenAccountOpts
from abstract_utilities import read_from_file,make_list
from construct import Bytes, Int8ul, Int64ul, BytesInteger
from solders.system_program import TransferParams, transfer
from spl.token.core import _TokenCore
from solana.rpc.commitment import Commitment
from spl.token.instructions import create_associated_token_account, get_associated_token_address
from abstract_bots.abstract_keypair import load_from_private_key,load_keypair_from_file
def get_lamports_per_sol():
    return 1000000000
AMM_PROGRAM_ID = Pubkey.from_string('675kPX9MHTjS2zt1qfr1NYHuzeLXfQM9H24wFSUt1Mp8')
SERUM_PROGRAM_ID = Pubkey.from_string('srmqPvymJeFKQ4zGQed1GFppgkRHL9kaELCbyksJtPX')
def get_account_params(base_public_key,new_public_key,fee,PublicKey,space=8):
        return sp.CreateAccountParams(
            from_pubkey=base_public_key,
            new_account_pubkey=new_public_key,
            lamports=fee,
            space=space,
            program_id=PublicKey(1),
        )

def get_keypair(key_pair_file):
    with open (key_pair_file) as kpf:
        keypair = kpf.read()
        keypair = keypair.replace('[', '').replace(']','')
        keypair = list(keypair.split(','))
        keypair = [int(i) for i in keypair]
    return Account(keypair[:32])

def get_account_info(client, public_key):
    account_info = client.get_account_info(public_key)
    print("--> Account Info: {}".format(account_info))
    balance = client.get_balance(public_key)
    print("--> Balance: {}".format(balance))
def get_ctx(rpc_url="https://api.mainnet-beta.solana.com",commitment="confirmed",timeout=30,blockhash_cache=True):
    cli=Client(rpc_url, commitment=Commitment(commitment), timeout=30,blockhash_cache=True)
    print("about to check connection")
    print("here is the connection: ", cli.is_connected())
    return cli
def pubkey_from_mint(mint):
    return Pubkey.from_string(mint)
def get_account_program_id_from_mint(mint,ctx=get_ctx()):
    pubkey = pubkey_from_mint(mint)
    return ctx.get_account_info_json_parsed(pubkey)

def get_owner(obj):
    return obj.value.owner
def get_program_id_from_mint(mint):
    pubkey = pubkey_from_mint(mint)
    program_id = get_program_id_from_pubkey(pubkey)
    return program_id

def get_account_program_id(pubkey,ctx=get_ctx()):
    return ctx.get_account_info_json_parsed(pubkey)

def get_program_id_from_pubkey(pubkey):
    accountProgramId=get_account_program_id(pubkey)
    return accountProgramId.value.owner

def get_accounts(payer,programid,ctx=get_ctx()):
    return ctx.get_token_accounts_by_owner_json_parsed(payer.pubkey()  ,TokenAccountOpts(program_id=programid)).value

def get_token_balance_lamports_from_account(account):
    return int(account.account.data.parsed['info']['tokenAmount']['amount'])

def get_mint_from_account(account):
    return account.account.data.parsed['info']['mint']

async def send_transaction(tx, signers, ctx):
    try:
        response = await ctx.send_transaction(tx, *signers)
        return response
    except Exception as e:
        print(f"Error sending transaction: {e}")
        return None

def get_recent_blockhash(ctx):
    return ctx.get_recent_blockhash()['result']['value']['blockhash']

def get_token_supply(mint_pubkey, ctx):
    return ctx.get_token_supply(mint_pubkey)['result']['value']['amount']

def get_token_account_balance(account_pubkey, ctx):
    return ctx.get_token_account_balance(account_pubkey)['result']['value']['amount']

async def get_associated_token_account_from_mint_keypair_file(mint,wallet_keypair):
    #if os.path.isfile(wallet_keypair):
    #    wallet_keypair = load_keypair_from_file(wallet_keypair)

    # Convert the mint address to a Pubkey object
    mint_pubkey = pubkey_from_mint(mint)

    # Get the associated token address for the wallet and the mint
    associated_token_address = await get_associated_token_address(
        wallet_keypair.pubkey(),  # The wallet public key
        mint_pubkey                 # The mint public key
    )
    return associated_token_address
"""
Check balance of the token in your wallet
"""
def checkB(TOKEN_TO_SWAP_SELL, payer, ctx):
    tokenPk = pubkey_from_mint(TOKEN_TO_SWAP_SELL)
    accountProgramId = get_account_program_id(tokenPk)
    programid = get_program_id_from_pubkey(tokenPk)
    accounts = get_accounts(payer,programid,ctx=get_ctx())
    for account in accounts:
        tokenBalanceLamports = get_token_balance_lamports_from_account(account)
        mint = get_mint_from_account(account)
        if tokenBalanceLamports > 0 and mint == TOKEN_TO_SWAP_SELL:
            return True
    return False
def load_from_private_key(env_key='AMM_P'):
    env_value = get_env_value(key=env_key)
    if env_value:
        return Keypair.from_base58_string(env_value)

def load_keypair_from_file(filename):
    curr = os.path.join(sys.path[0], 'data',  filename)
    with open(curr, 'r') as file:
        secret = json.load(file)
        secret_key = bytes(secret)
        # print(base58.b58encode(secret_key))
        return Keypair.from_bytes(secret_key)


def make_swap_instruction(amount_in: int, token_account_in: Pubkey.from_string, token_account_out: Pubkey.from_string,
                              accounts: dict, mint, ctx, owner) -> Instruction:
        tokenPk = Pubkey.from_string(mint)
        TOKEN_PROGRAM_ID = get_program_id_from_mint(mint)
        accounts=fetch_pool_keys(mint)
        keys = [
            AccountMeta(pubkey=TOKEN_PROGRAM_ID, is_signer=False, is_writable=False),
            AccountMeta(pubkey=accounts["amm_id"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["authority"], is_signer=False, is_writable=False),
            AccountMeta(pubkey=accounts["open_orders"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["target_orders"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["base_vault"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["quote_vault"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=SERUM_PROGRAM_ID, is_signer=False, is_writable=False), 
            AccountMeta(pubkey=accounts["market_id"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["bids"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["asks"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["event_queue"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["market_base_vault"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["market_quote_vault"], is_signer=False, is_writable=True),
            AccountMeta(pubkey=accounts["market_authority"], is_signer=False, is_writable=False),
            AccountMeta(pubkey=token_account_in, is_signer=False, is_writable=True),  #UserSourceTokenAccount 
            AccountMeta(pubkey=token_account_out, is_signer=False, is_writable=True), #UserDestTokenAccount 
            AccountMeta(pubkey=owner.pubkey(), is_signer=True, is_writable=False) #UserOwner 
        ]

        data = get_swap_layout().build(
            dict(
                instruction=9,
                amount_in=int(amount_in),
                min_amount_out=0
            )
        )
        return Instruction(AMM_PROGRAM_ID, data, keys)

def get_token_account(ctx, 
                      owner: Pubkey.from_string, 
                      mint: Pubkey.from_string):
    try:
        account_data = ctx.get_token_accounts_by_owner(owner, TokenAccountOpts(mint))
        return account_data.value[0].pubkey, None
    except:
        swap_associated_token_address = get_associated_token_address(owner, mint)
        swap_token_account_Instructions = create_associated_token_account(owner, owner, mint)
        return swap_associated_token_address, swap_token_account_Instructions

def sell_get_token_account(ctx, 
                      owner: Pubkey.from_string, 
                      mint: Pubkey.from_string):
    try:
        account_data = ctx.get_token_accounts_by_owner(owner, TokenAccountOpts(mint))
        return account_data.value[0].pubkey
    except:
        print("Mint Token Not found")
        return None


def extract_pool_info(pools_list: list, mint: str) -> dict:
    for pool in pools_list:
        
        if pool['baseMint'] == mint and pool['quoteMint'] == 'So11111111111111111111111111111111111111112':
            return pool
        elif pool['quoteMint'] == mint and pool['baseMint'] == 'So11111111111111111111111111111111111111112':
            return pool
    raise Exception(f'{mint} pool not found!')


def fetch_pool_keys(mint: str):
    amm_info = {}
    all_pools = {}
    try:
        # Using this so it will be faster else no option, we go the slower way.
        with open('all_pools.json', 'r') as file:
            all_pools = json.load(file)
        amm_info = extract_pool_info(all_pools, mint)
    except:
        resp = requests.get('https://api.raydium.io/v2/sdk/liquidity/mainnet.json', stream=True)
        pools = resp.json()
        official = pools['official']
        unofficial = pools['unOfficial'] 
        all_pools = official + unofficial

        # Store all_pools in a JSON file
        with open('all_pools.json', 'w') as file:
            json.dump(all_pools, file)
        try:
            amm_info = extract_pool_info(all_pools, mint)
        except:
            return "failed"
        
    return {
    'amm_id': Pubkey.from_string(amm_info['id']),
    'authority': Pubkey.from_string(amm_info['authority']),
    'base_mint': Pubkey.from_string(amm_info['baseMint']),
    'base_decimals': amm_info['baseDecimals'],
    'quote_mint': Pubkey.from_string(amm_info['quoteMint']),
    'quote_decimals': amm_info['quoteDecimals'],
    'lp_mint': Pubkey.from_string(amm_info['lpMint']),
    'open_orders': Pubkey.from_string(amm_info['openOrders']),
    'target_orders': Pubkey.from_string(amm_info['targetOrders']),
    'base_vault': Pubkey.from_string(amm_info['baseVault']),
    'quote_vault': Pubkey.from_string(amm_info['quoteVault']),
    'market_id': Pubkey.from_string(amm_info['marketId']),
    'market_base_vault': Pubkey.from_string(amm_info['marketBaseVault']),
    'market_quote_vault': Pubkey.from_string(amm_info['marketQuoteVault']),
    'market_authority': Pubkey.from_string(amm_info['marketAuthority']),
    'bids': Pubkey.from_string(amm_info['marketBids']),
    'asks': Pubkey.from_string(amm_info['marketAsks']),
    'event_queue': Pubkey.from_string(amm_info['marketEventQueue'])
        }

def get_pool_info_layout():
    return cStruct(
    "instruction" / Int8ul,
    "simulate_type" / Int8ul
    )

def get_swap_layout():
    return cStruct(
    "instruction" / Int8ul,
    "amount_in" / Int64ul,
    "min_amount_out" / Int64ul
    )

# Not in use right now, might be useful in future
def get_amm_info_layout_4():
    return cStruct(
    'status' / Int64ul,
    'nonce' / Int64ul,
    'order_num' / Int64ul,
    'depth' / Int64ul,
    'base_decimal' / Int64ul,
    'quote_decimal' / Int64ul,
    'state' / Int64ul,
    'reset_flag' / Int64ul,
    'min_size' / Int64ul,
    'vol_max_cut_ratio' / Int64ul,
    'amount_wave_ratio' / Int64ul,
    'base_lot_size' / Int64ul,
    'quote_lot_size' / Int64ul,
    'min_price_multiplier' / Int64ul,
    'max_price_multiplier' / Int64ul,
    'system_decimal_value' / Int64ul,
    # Fees
    'min_separate_numerator' / Int64ul,
    'min_separate_denominator' / Int64ul,
    'trade_fee_numerator' / Int64ul,
    'trade_fee_denominator' / Int64ul,
    'pnl_numerator' / Int64ul,
    'pnl_denominator' / Int64ul,
    'swap_fee_numerator' / Int64ul,
    'swap_fee_denominator' / Int64ul,
    # OutPutData
    'base_need_take_pnl' / Int64ul,
    'quote_need_take_pnl' / Int64ul,
    'quote_total_pnl' / Int64ul,
    'base_total_pnl' / Int64ul,
    # 128
    'quote_total_deposited' / BytesInteger(16, signed=False, swapped=True),
    'base_total_deposited' / BytesInteger(16, signed=False, swapped=True),
    'swap_base_in_amount' / BytesInteger(16, signed=False, swapped=True),
    'swap_quote_out_amount' / BytesInteger(16, signed=False, swapped=True),

    'swap_base2_quote_fee' / Int64ul,
    # 128
    'swap_quote_in_amount' / BytesInteger(16, signed=False, swapped=True),
    'swap_base_out_amount' / BytesInteger(16, signed=False, swapped=True),

    'swap_quote2_base_fee' / Int64ul,
    # AMM Vault
    'base_vault' / Bytes(32),
    'quote_vault' / Bytes(32),
    # Mint
    'base_mint' / Bytes(32),
    'quote_mint' / Bytes(32),
    'lp_mint' / Bytes(32),
    # Market
    'open_orders' / Bytes(32),
    'market_id' / Bytes(32),
    'serum_program_id' / Bytes(32),
    'target_orders' / Bytes(32),
    'withdraw_queue' / Bytes(32),
    'lp_vault' / Bytes(32),
    'amm_owner' / Bytes(32),

    'lpReserve' / Int64ul,
    )



import os
import re
import json
import shutil
import logging
from abstract_ai import ApiManager
from abstract_ai import ModelManager
logging.basicConfig(level=logging.INFO)
logging.basicConfig(level=logging.DEBUG,format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)
from abstract_utilities import (safe_dump_to_file,
                                safe_read_from_json,
                                get_any_value,
                                make_list,
                                eatAll)
#json functions-----------------------------------------------------------------------------------------------------------------------------------------
def safe_json_loads(data):
    if not isinstance(data, (dict, str)):
        data = str(data)
    if not isinstance(data, dict):
        try:
            data = json.loads(data)
        except json.JSONDecodeError as e:
            logger.error(f"type: Error\nfunction: safe_json_loads \nscript: bot_functions\n data: {data}\nerror: {e}")
            data = None
    return data

def load_json_data(file_path):
    try:
        with open(file_path,'r') as file:
            return json.load(file)
    except json.JSONDecodeError as e:
        logger.error(f"type: Error\nfunction: load_json_data \nscript: bot_functions\n data: {file_path}\nerror: {e}")
        return None
    
def safe_list(obj,keys):
    keys = make_list(keys)
    for key in keys:
        if obj and isinstance(obj,list) and is_number(key) and len(obj)>key:
            obj = obj[int(key)]
        else:
            return obj
    return obj

def safe_get(obj,keys):
    keys = make_list(keys)
    new_obj = obj
    for i,key in enumerate(keys):
        if isinstance(new_obj,dict):
            new_obj = new_obj.get(key,None if len(keys)-1 == i else {})
        elif isinstance(new_obj,list):
            new_obj = safe_list(new_obj,key)
        else:
            return new_obj
    return new_obj

def get_all_keys(dict_data,keys=[]):
  if isinstance(dict_data,dict):
    for key,value in dict_data.items():
      keys.append(key)
      keys = get_all_keys(value,keys=keys)
  return keys
#address sanitation-----------------------------------------------------------------------------------------------
def get_data(file_path):
    if os.path.isfile(file_path):
        return safe_read_from_json(file_path)
    
#directory functions --------------------------------------------------------------------------------------------
def move_file(src_path,dst_file_path):
    shutil.move(str(src_path), str(dst_file_path))
    logger.info(f"Moved {src_path} to errored messages directory.")

def secure_delete(file_path):
    try:
        if os.path.exists(file_path):
            os.remove(file_path)
        else:
            logger.warning(f"type: warning\nfunction: secure_delete \nscript: bot_functions\n data: {file_path}\nerror: does not exist.")
    except OSError as e:
        logger.error(f"type: Error\nfunction: secure_delete \nscript: bot_functions\n data: {file_path}\nerror: {e.strerror}")

def make_directory(file_path):
    os.makedirs(file_path, exist_ok=True)
    return file_path

def get_create_data(file_path,data={}):
    if not os.path.isfile(file_path):
        safe_dump_to_file(data=data,file_path=file_path)
    return safe_read_from_json(file_path)

#type functions --------------------------------------------------------------------------------------------------
def get_args_send_args(function, *args):
    if not args:
        return args
    input_type = type(args[0])
    flattened_args = []
    for arg in args:
        if isinstance(arg, (list, set, tuple)):
            flattened_args.extend(arg)
        else:
            flattened_args.append(arg)
    processed_items = [function(item) for item in flattened_args]
    if input_type is list:
        return processed_items
    elif input_type is set:
        return set(processed_items)
    elif input_type is tuple:
        return tuple(processed_items)
    else:
        # Assuming if_single_obj is a function that handles single objects
        return if_single_obj(processed_items)
    
def list_set(obj):
    obj = obj or []
    return list(set(obj))

def get_symetric_difference(obj_1,obj_2):
    set1 = set(obj_1)
    set2 = set(obj_2)
    # Find elements that are unique to each list
    unique_elements = set1.symmetric_difference(set2)
    # Convert the set back to a list, if needed
    return list(unique_elements)

def if_single_obj(list_obj):
    if list_obj and isinstance(list_obj,list) and len(list_obj)==1:
        list_obj = list_obj[0]
    return list_obj

def list_set(obj):
    try:
        obj = list(set(obj))
    except Exception as e:
        print(f"{e}")
    return obj

def str_lower(obj):
    try:
        obj=str(obj).lower()
    except Exception as e:
        print(f"{e}")
    return obj

def is_number(obj):
    try:
        float(obj)
        return True
    except (ValueError, TypeError):
        return False
#address functions-------------------------------------------------------------------------------------------------
def normalize_address(address):
    address =str(address).lower()
    if address.startswith('0x'):
        address = address[2:]
    return address

def normalize_all_addresses(*args):
    return get_args_send_args(normalize_address,*args)

def serialize_all_addresses(*args):
    return get_args_send_args(check_and_reserialize_solana_address, *args)

def make_directory(data_path):
    os.makedirs(data_path, exist_ok=True)
    return data_path

def get_amount_dict(amount,decimals=9):
    if amount!= None:
        if isinstance(amount,dict):
            amount_dict = get_any_value(amount,'uiTokenAmount')
            amount = get_any_value(amount_dict,'amount')
            decimals = get_any_value(amount_dict,'decimals')
        return exponential(amount,decimals,-1)
#math functions ------------------------------------------------------------------------------------------------------
def exponential(value,exp=9,num=-1):
    return multiply_it(value,exp_it(10,exp,num))

def return_0(*args):
    for arg in args:
        if arg == None or not is_number(arg) or arg in [0,'0','','null',' ']:
            return float(0)
        
def exp_it(number,integer,mul):
    if return_0(number,integer,mul)==float(0):
        return float(0)
    return float(number)**float(float(integer)*int(mul))

def divide_it(number_1,number_2):
    if return_0(number_1,number_2)==float(0):
        return float(0)
    return float(number_1)/float(number_2)

def multiply_it(number_1,number_2):
    if return_0(number_1,number_2)==float(0):
        return float(0)
    return float(number_1)*float(number_2)

def add_it(number_1,number_2):
    if return_0(number_1,number_2)==float(0):
        return float(0)
    return float(number_1)+float(number_2)

def get_percentage(owner_balance,address_balance):
    retained_div = divide_it(owner_balance,address_balance)
    retained_mul = multiply_it(retained_div,100)
    return round(retained_mul,2)

 




from .bot_functions import get_any_value,safe_get,is_number
from .txn_functions import get_account_keys
def get_supply_from_raw(raw_supply):
    value = get_any_value(raw_supply,'value')
    amount = get_any_value(value,'uiAmount')
    return amount or value or raw_supply
def get_token(amount):
    if amount:
        token = 'So11111111111111111111111111111111111111112'
        if isinstance(amount,dict):
            token = safe_get(amount,['mint'])
        return token
def is_within_range(i,value,comp_value):
  if value and is_number(value):
    small_value = float(value)-float(i)
    big_value = float(value)+float(i)
    if comp_value and is_number(comp_value):
      comp_value = float(comp_value)
      if comp_value>=small_value and comp_value<=big_value:
        return True
def get_price_if_avaliable(comp_address,price_data,block_time,time_range=5):
  prices = {}
  for address,values in price_data.items():
    for value in [int(value) for value in list(values["prices"].keys()) if is_within_range(time_range,block_time,value)]:
        if comp_address in values["prices"][str(value)].keys():
          
          if address not in prices:
            prices[address]=[]
          prices[address]+=values["prices"][str(value)][comp_address]["prices"]
  if prices:
    for address,pricing in prices.items():
      pricing = [price for price in prices[address] if float(price) != float(0)]
      if pricing:
        prices[address] = sum(pricing)/len(pricing)
      else:
        prices[address] = pricing
  return prices
def get_names(price_data,mint,knowns):
    if mint in knowns.values() and price_data[mint]["name"] == None:
        for name,value in knowns.items():
          if value == mint:
            price_data[mint]["name"]=name
    return price_data
def get_mint(price_data,mint):
    if mint not in price_data:
        price_data[mint]={"name":None,"chain":"solana","signatures":[],"prices":{}}
    return price_data
def get_block_time(price_data,mint,mint_2,blockTime):
    if blockTime not in price_data[mint]["prices"]:
        price_data[mint]["prices"][blockTime] = {}
    return price_data
def get_block_time_data(price_data,mint,mint_2,blockTime):
    if mint_2 not in price_data[mint]["prices"][blockTime]:
        price_data[mint]["prices"][blockTime][mint_2] = {"price":None,"prices":[]}
    return price_data
def get_signature_price(price_data,mint,mint_2,tokens,blockTime,signature):
    if signature not in price_data[mint]["signatures"]:
        price_data[mint]["prices"][blockTime][mint_2]["prices"].append(abs(float(tokens[-1][mint]['change'])/float(tokens[-1][mint_2]['change'])))
        price_data[mint]["signatures"].append(signature)
    return price_data
def get_prices(mint,blockTime,address):
    return price_data[mint]["prices"][blockTime][address]['price']
def print_token_amt(signature_data,i):
  account_keys = get_account_keys(signature_data)
  preTokens = get_any_value(signature_data,'preTokenBalances')
  postTokens = get_any_value(signature_data,'postTokenBalances')
  postTokens = [index for index in postTokens if index.get('accountIndex') == i]
  preTokens = [index for index in preTokens if index.get('accountIndex') == i]
  if postTokens and preTokens:
    account=None
    postAccount=None
    preAccount=None
    preToken =preTokens[0]
    postToken=postTokens[0]
    pre_balance = get_any_value(preToken,'uiAmount')
    post_balance = get_any_value(postToken,'uiAmount')
    if pre_balance != None and post_balance != None:
      change = post_balance-pre_balance 
      postOwner = postToken.get('owner')
      preOwner = preToken.get('owner')
      postProgramId = postToken.get('programId')
      preProgramId = preToken.get('programId')
      postAccountIndex=postToken.get('accountIndex')
      preAccountIndex=preToken.get('accountIndex')
      postMint=postToken.get('mint')
      preMint=preToken.get('mint')

      if preAccountIndex != None:
        preAccount=account_keys[preAccountIndex]
      if postAccountIndex != None:
        postAccount=account_keys[postAccountIndex]
      
      if change:
        #print(f'\nchange {change}\npostOwner={postOwner}\npostProgramId={postProgramId}\npostAccount={postAccount}\npostMint={postMint}\n')
        #input()
        return [postMint,change]
def get_usd_price(price_data,signature_data,knowns):
    #print(f"\n\nsignature {signature}")
    tokens = [{}]
    account_keys = get_account_keys(signature_data)
    signature = safe_get(get_any_value(signature_data,'signatures'),[0])
    blockTime = get_any_value(signature_data,'blockTime')
    postBalances = get_any_value(signature_data,'postBalances')
    preBalances = get_any_value(signature_data,'preBalances')
    fee = get_any_value(signature_data,'fee')
    #if fee:
    #print(f"fee = {fee*(10**(-9))}")
    for j,preBalance in enumerate(preBalances):
        sol_change = postBalances[j] - preBalance
        token = print_token_amt(signature_data,j)
        if token:
            keys = list(tokens[-1].keys())
            if tokens[0] not in keys:
                tokens[-1][token[0]]={"change":token[1],"price":{},"prices":{}}
            if len(keys) == 2:
                for key_num,key in enumerate(keys):
                    key_now = key_num-1
                    mint = keys[key_num]
                    mint2=keys[key_now]
                    price_data = get_mint(price_data,mint)
                    price_data = get_names(price_data,mint,knowns)
                    price_data = get_block_time(price_data,mint,mint2,blockTime)
                    price_data = get_block_time_data(price_data,mint,mint2,blockTime)
                    price_data = get_signature_price(price_data,mint,mint2,tokens,blockTime,signature)
                    #print(f"{price_data[keys[key_num]]['name'] or keys[key_num]} worth:\n")
                    for address,price in get_price_if_avaliable(keys[key_num],price_data,blockTime).items():
                        #input(price_data[keys[key_num]]["prices"][blockTime])
                        if price:                  
                            if address not in price_data[keys[key_num]]["prices"][blockTime]:
                                price_data[keys[key_num]]["prices"][blockTime][address]={'price':None,'prices':[]}
                            if price_data[keys[key_num]]["prices"][blockTime][address]['price'] == None:
                                price_data[keys[key_num]]["prices"][blockTime][address]['price'] = price
                            else:
                                price_data[keys[key_num]]["prices"][blockTime][address]['price'] = float(price_data[keys[key_now]]["prices"][blockTime][keys[key_num]]['price'] + price)      
    return price_data



import os
import requests
import threading
def make_directory(file_path):
    os.makedirs(file_path,exist_ok=True)
    return file_path
class SingletonMeta(type):
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(SingletonMeta, cls).__call__(*args, **kwargs)
        return cls._instances[cls]
class directoryStart(metaclass=SingletonMeta):
    def __init__(self):
        if not hasattr(self, 'initialized'):  # Prevent reinitialization
            self.initialized = True
            self.abs_endpoint = os.path.abspath(__file__)
            self.src_dir = os.path.dirname(self.abs_endpoint)
            #data folder
            self.data_path = make_directory(os.path.join(self.src_dir, 'data'))
            #accounts
            self.rpc_call_dir = make_directory(os.path.join(self.data_path,'rpc_call'))
directory_mgr = directoryStart()
def get_client_rpc(net_type="main"):
    net_type == {"main":"mainnet-beta","dev":"devnet","test":"testnet","default":"http://192.168.1.88:8899"}.get(net_type,"mainnet-beta")
    return f"https://api.{net_type}.solana.com"
def get_unfiltered_socket_directory():
    return directory_mgr.unfiltered_socket_data_dir
def save_unfiltered_socket_data(slot,signature,data):
    socket_directory = get_unfiltered_socket_directory()
    slot_directory = make_directory(os.path.join(socket_directory,str(slot)))
    signature_file_path = os.path.join(slot_directory,f"{signature}.json")
    safe_dump_to_file(file_path = signature_file_path, data=data)
class rpcCallManager(metaclass=SingletonMeta):
    def __init__(self,directory_mgr=None,endpoint = get_client_rpc(net_type="main")):
        if not hasattr(self, 'initialized'):  # Prevent reinitialization
            self.initialized = True
            self.endpoint = endpoint
            self.directory_mgr = directory_mgr or directoryStart()
            self.rpc_call_type_folder = self.directory_mgr.rpc_call_dir
            self.recently_checked_signature_data={}
    def get_result(self,response):
        try:
            response = response.json()
        except:
            return response.text
        if isinstance(response,dict):
            keys = response.keys()
            if 'result' in keys:
                return response['result']
        return response
    def get_conditions(self,conditions):
        # Remove None values to avoid sending them as part of the request
        return {k: v for k, v in conditions.items() if v is not None}
    def get_response(self,call_type, params):
        response = self.rpc_call(call_type, params)
        if isinstance(response,dict) and response.get('error') != None:
            return response
        data = {"call_type":call_type,"params":params,"response":response}
  
        file_path = self.get_file_name(call_type)
        safe_dump_to_file(file_path=file_path,data=data)
        return response
    def get_file_path(self,call_type,file):
        return os.path.join(self.get_call_folder(call_type),file)
    def get_call_folder(self,call_type):
        return make_directory(os.path.join(self.rpc_call_type_folder,call_type))
    def get_file_name(self, call_type):
        timestamp = datetime.now().strftime("%Y%m%d%H%M%S%f")
        file_name_try = f"response_{timestamp}.json"
        rpc_call_type_folder = self.get_call_folder(call_type)
        file_path = os.path.join(rpc_call_type_folder, file_name_try)
        return file_path
    def await_statuse(self,rpc_url,headers,request_data):
        for i in range(3):
            response = requests.post(url=rpc_url, data=json.dumps(request_data), headers=headers)
            if i >1:
                print(f"response loop has gone for {i} itterations; the current response is: {response}")
            if response.status_code == 200:
                return self.get_result(response)
            elif response.status_code == 429:
                # If a 429 status code is received, wait and retry
                wait_time = int(response.headers.get("Retry-After", 5))  # Default wait time is 5 seconds
                time.sleep(wait_time)
        return response
    def rpc_call(self,method, params):
        rpc_url = rpc
        headers = {"Content-Type": "application/json"}
        request_data = {
            "jsonrpc": "2.0",
            "id": 1,
            "method": method,
            "params": params,
        }
        return self.await_statuse(self.endpoint,headers,request_data)
    def params_match(self,saved_params, current_params):
        if len(saved_params) != len(current_params):
            return False
        for saved, current in zip(saved_params, current_params):
            if type(saved) != type(current):
                return False
            if isinstance(saved, dict):
                if saved != current:
                    return False
            elif isinstance(saved, list):
                if sorted(saved) != sorted(current):
                    return False
            else:
                if saved != current:
                    return False
        return True
    def json_rpc_call(self,call_type,params):
        
        for file in os.listdir(self.get_call_folder(call_type)):
            file_path = self.get_file_path(call_type,file)
            data = safe_read_from_json(file_path)
            
            if isinstance(data,dict):
                parameters = data.get('params')
                same = self.params_match(parameters, params)
                if same:
                    if data['response']:
                        return data['response']
        return self.get_response(call_type, params)
    def get_transaction(self,slot,signature):
        signature_data = block_data_mgr.get_block_data_signature_data(slot, signature)
        if signature_data:
            return signature_data
        signature_data = self.get_response("getTransaction", [signature, {"maxSupportedTransactionVersion": 0}])
        block_data_mgr.save_block_data_signature_data(slot, signature, signature_data)
        return signature_data

    def fetch_account_data(self,account):
        account_data = self.json_rpc_call("AddressLookupTableAccount",[account, {"maxSupportedTransactionVersion": 0}])
        return account_data

    def get_signature_for_address(self, account=None, address=None, limit=10, before=None, after=None, finalized=True):
        # Prepare the conditions for the RPC call
        account = account or address
        conditions = {
            "limit": limit,
            "before": before,
            "after": after,
            "finalized": finalized,
        }
        # Remove None values to avoid sending them as part of the request
        conditions = {k: v for k, v in conditions.items() if v is not None}
        
        # Perform the RPC call
        return self.json_rpc_call("getSignaturesForAddress", [account, conditions])      
    def get_account_info(self, account=None, address=None, encoding='jsonParsed'):
        account = account or address
        conditions = {
            "encoding":encoding
            }
        return rpc_mgr.json_rpc_call("getAccountInfo",[address,self.get_conditions(conditions)])
    def get_token_account_balance(self, account=None, address=None, mint=None):
        account = account or address
        conditions = {
            "mint":mint,
            "commitment":0
            }

        return rpc_mgr.json_rpc_call("getTokenAccountBalance",[account,self.get_conditions(conditions)])
    def get_token_accounts_by_delegate(self, account=None, address=None, programId=None,encoding='jsonParsed'):
        account = account or address
        conditions = {
            "programId":programId,
            "encoding":encoding
            }
        return rpc_mgr.json_rpc_call("getTokenAccountsByDelegate",[account,self.get_conditions(conditions)])
    def get_account_info(self, account=None, address=None, encoding='jsonParsed'):
        account = account or address
        conditions = {
            "encoding":encoding
            }
        return rpc_mgr.json_rpc_call("getAccountInfo",[account,self.get_conditions(conditions)])
    def get_token_accounts_by_owner(self, account=None, address=None, mint=None,encoding='jsonParsed'):
        account = account or address
        conditons = {
            "mint":mint,
            "encoding": encoding
            }
        return rpc_mgr.json_rpc_call("getTokenAccountsByOwner",[account,self.get_conditions(conditons)])
    

    def get_token_supply(self,token):
        return self.json_rpc_call("getTokenSupply",[token])
    def get_block_time_from_slot(self,slot):
        return self.json_rpc_call("getBlockTime",[int(slot)])
class assetRequestManager(metaclass=SingletonMeta):
    def __init__(self):
        if not hasattr(self, 'initialized'):  # This check is necessary to avoid reinitialization in a singleton
            self.initialized = True
            self.call_history_manager = {
                'dexscreener': {"last_call": None, 'last_call_time': 0},
                'solana': {"last_call": None, 'last_call_time': 0}
            }
            self.epoch_tracker={}
            self.default_calls_per_min = 10
            self.calls_per_min = {"dexscreener": 300, "solana": self.default_calls_per_min}
            self.lock = threading.Lock()
    def epoch_check(self,api_type):
        if api_type not in self.calls_per_min:
            self.calls_per_min[api_type] = self.default_calls_per_min
        if api_type not in self.epoch_tracker:
            self.epoch_tracker[api_type]=[]
        current_time = time.time()
        epoch_start = current_time-60
        new_epoch_tracker = []
        for i,call in enumerate(self.epoch_tracker[api_type]):
            if call >=epoch_start:
                new_epoch_tracker = self.epoch_tracker[api_type][i:]
                break
        self.epoch_tracker[api_type]=new_epoch_tracker
        if len(self.epoch_tracker[api_type])>=self.calls_per_min[api_type]:
            return 60 - (time.time() - self.epoch_tracker[api_type][0])
        return 0
    def get_rest_time(self, api_type):
        sleep_time = self.epoch_check(api_type)
        if sleep_time > 0:
            time.sleep(sleep_time)

    def dexscreener_api_call(self, url):
        base_url = os.path.basename(url)
        identifier_url = os.path.basename(os.path.dirname(url))
        with self.lock:
            if base_url not in self.call_history_manager['dexscreener']:
                self.call_history_manager['dexscreener'][base_url] = {}
            if identifier_url not in self.call_history_manager['dexscreener'][base_url]:
                self.call_history_manager['dexscreener'][base_url][identifier_url] = {'time': 0, 'response': None, 'status_code': None}
        
        thread = threading.Thread(target=self._dexscreener_api_call_thread, args=(url,))
        thread.start()
        thread.join()
        return self._get_response_from_history('dexscreener', base_url, identifier_url)

    def _dexscreener_api_call_thread(self, url):
        base_url = str(os.path.basename(url))
        initial_url = os.path.dirname(url)
        identifier_url = os.path.basename(initial_url)
        with self.lock:
            if base_url not in self.call_history_manager['dexscreener']:
                self.call_history_manager['dexscreener'][base_url] = {}
            if identifier_url not in self.call_history_manager['dexscreener'][base_url]:
                self.call_history_manager['dexscreener'][base_url][identifier_url] = {"time": 0, "url": url, "response": None, "status_code": None}
            if float(time.time()) - float(self.call_history_manager['dexscreener'][base_url][identifier_url]['time']) > 30 or self.call_history_manager['dexscreener']["last_call"] != [base_url, identifier_url]:
                self.get_rest_time('dexscreener')
                response,status_code = make_request(url,'pairs',status_code=True)
                self.epoch_tracker['dexscreener'].append(time.time())
                self.call_history_manager['dexscreener']["last_call"] = [str(base_url), str(identifier_url)]
                self.call_history_manager['dexscreener'][base_url][identifier_url]['time'] = self.epoch_tracker['dexscreener'][-1]
                self.call_history_manager['dexscreener'][base_url][identifier_url]['status_code'] = status_code
                self.call_history_manager['dexscreener'][base_url][identifier_url]['response'] = response
    def _get_response_from_history(self, api_type, base_url, identifier_url):
        with self.lock:
            return self.call_history_manager[api_type][base_url][identifier_url]['response']
def api_call_token(token):
    url = f"https://api.dexscreener.com/latest/dex/tokens/{token}"
    return request_mgr.dexscreener_api_call(url)
def api_call_pairs(token):
    url = f"https://api.dexscreener.com/latest/dex/pairs/solana/{token}"
    return request_mgr.dexscreener_api_call(url)
def call_token(address):
    token_info = api_call_token(address)
    if token_info:
        return 'token',token_info
    return 'token',None
def call_pair(address):
    token_info = api_call_pairs(address)
    if token_info:
        return 'pair',token_info
    return 'pair',None
def get_token_info(address,address_type='token'):
    address_type = {'pair':'pair','token':'token'}.get(address_type,'token')
    call = [address_type,{'pair':'token','token':'pair'}.get(address_type)]
    for call_type in call:
        if call_type == 'pair':
            address_type,address_info = call_pair(address)
        elif call_type == 'token':
            address_type,address_info = call_token(address)
        if address_info:
            return address_type,address_info
    return 'address',address_info
rpc_mgr = rpcCallManager()
asset_mgr = assetRequestManager()



from .bot_functions import safe_get,get_any_value,exponential
def search_for_account_index(data,index_number):
    for index_data in data:
        if str(safe_get(index_data,['accountIndex'])) == str(index_number):
            return index_data
def get_amount_dict(amount,decimals=9):
    if amount!= None:
        if isinstance(amount,dict):
            amount_dict = get_any_value(amount,'uiTokenAmount')
            amount = get_any_value(amount_dict,'amount')
            decimals = get_any_value(amount_dict,'decimals')
        return exponential(amount,decimals,-1)
def calculate_change(pre,post,decimals=9):
    if pre !=None and post!=None:
        return get_amount_dict(pre,decimals) - get_amount_dict(post,decimals)
def get_acct_index(txn_details,acct_index):
    for each in txn_details:
        if each['accountIndex'] == acct_index:
            return each
def get_account_keys(signature_data):
    accountKeys = get_any_value(signature_data,'accountKeys')
    loaded_addresses = get_any_value(signature_data,'loadedAddresses')
    for key,value in loaded_addresses.items():
        accountKeys+=value
    return accountKeys
def create_token_txns(signature_data):
    preTokenBalances = get_any_value(signature_data,'preTokenBalances')
    postTokenBalances = get_any_value(signature_data,'postTokenBalances')
    account_keys = get_account_keys(signature_data)
    txns=[]
    dont_use = []
    all_txns = {"Address":[],"Owner":[],'Balance Before':[],"Balance After":[],"Change":[],"Token":[]}
    for i,each in enumerate([preTokenBalances,postTokenBalances]):
        for post in each:
            index = post['accountIndex']
            if index not in dont_use:
                dont_use.append(index)
                after = get_amount_dict(get_acct_index(postTokenBalances,index))
                change = get_amount_dict(post)
                if after!=None and change !=None:
                    before = after-change
                    if i == 0:
                        pre_change = change
                        change = before
                        before = pre_change
                    all_txns["Address"].append(safe_get(account_keys,index))
                    all_txns["Owner"].append(safe_get(post,['owner']))
                    all_txns['Balance Before'].append(before)
                    all_txns["Balance After"].append(after)
                    all_txns["Change"].append(change)
                    all_txns["Token"].append(safe_get(post,['mint']))
                    #asset_mgr.return_token_info(address=post['mint'], address_type="token")
    return all_txns
def get_solana_balances(signature_data):
    balance = []
    preBalances = get_any_value(signature_data,'preBalances')
    postBalances = get_any_value(signature_data,'postBalances')
    account_keys = get_account_keys(signature_data)
    all_txns = {"Address":[],'Balance Before':[],"Balance After":[],"Change":[]}
    for i, amt in enumerate(preBalances):
        balance_change = [exponential(amt, 9,-1), -exponential(postBalances[i], 9,-1)]
        balance.append(balance_change)
          
        address = account_keys[i]
        all_txns['Address'].append(address)
        #asset_mgr.return_token_info(address=address, address_type="account")
        all_txns['Balance Before'].append(balance[-1][0])
        all_txns['Balance After'].append(balance[-1][1])
        all_txns['Change'].append(sum(balance[-1]))
    return all_txns
def get_all_txns(signature_data):
    output_js={}
    result = get_solana_balances(signature_data)
    output_js['solana']=result
    result = create_token_txns(signature_data)
    output_js['Token']=result
    return output_js
def parse_pool_data(logMessages):
    log_messages_string = str(get_any_value(logMessages,'logMessages') or logMessages)
    new_js={}
    init_split = log_messages_string.split('InitializeInstruction2 {')[1]
    brac_split = init_split.split('}')[0].replace(' ','')
    for each in brac_split.split(','):
        each=each.split(':')
        new_js[each[0]] = each[1]
    return new_js



import os
import json
import base64
import base58
from solana.rpc.api import Client
from solders.pubkey import Pubkey
from solders.keypair import Keypair
from solders.signature import Signature
from solana.rpc.commitment import Commitment
from solana.rpc.types import TokenAccountOpts
from abstract_security.envy_it import get_env_value
from abstract_utilities import read_from_file,make_list
from abstract_utilities import write_to_file,safe_dump_to_file,safe_read_from_json
wallet_address = "675kPX9MHTjS2zt1qfr1NYHuzeLXfQM9H24wFSUt1Mp8"
wss = "wss://rpc.ankr.com/solana/ws/e1fb8a495a70e554ea5ac00c694c5c3f33ce28d076ecb8dc46c2df3a4e03f432"
rpc="https://rpc.ankr.com/solana/e1fb8a495a70e554ea5ac00c694c5c3f33ce28d076ecb8dc46c2df3a4e03f432"
solana_client = Client(rpc)
ctx = Client(rpc, commitment=Commitment("confirmed"), timeout=30, blockhash_cache=True)
def deserialize_data(data):
    # Example of deserializing data. This needs to be tailored to the specific serialization format.
    # For instance, if it's base58 encoded, you can use base58 library to decode it.
    # If it's Borsh serialized, you'd use a Borsh deserialization process.
    try:
        decoded_data = base58.b58decode(data)
        return decoded_data
    except Exception as e:
        print(f" could not get base58 from {data}:\n{e}")
        return data
def deserialize_base64_data(data):
    # This function is for deserializing data encoded in base64.
    try:
        # Decode the base64 data
        decoded_data = base64.b64decode(data)
        return decoded_data
    except Exception as e:
        print(f"Could not decode base64 data from {data}:\n{e}")
        return None
def correct_and_decode_base64(data):
    try:
        # Corrects any padding issues by adding the necessary padding
        padding = 4 - (len(data) % 4)
        data += "=" * padding
        return base64.b64decode(data)
    except Exception as e:
        print(f"Could not decode base64 data: {e}")
        return None

def load_from_private_key(env_key='AMM_P'):
    env_value = get_env_value(key=env_key)
    if env_value:
        return Keypair.from_base58_string(env_value)
    
def get_user_pub_key():
    return load_from_private_key('AMM_P').pubkey()

def load_keypair_from_file(filename):
    curr = os.path.join(sys.path[0], 'data',  filename)
    with open(curr, 'r') as file:
        secret = json.load(file)
        secret_key = bytes(secret)
        # print(base58.b58encode(secret_key))
        return Keypair.from_bytes(secret_key)
    
def get_pub_key(address):
    if pubkey_type() != type(address):
        try:
            tokenPk = Pubkey.from_string(address)
            return tokenPk
        except Exception as e:
            print("public key could not be derived: {e}")
            return None
    return address

def pubkey_type():
    return type(Pubkey.from_string("6fvys6PkSCgZcrDCCkYp56BjLMKZ41ySZK6qtgzX49Hg"))

def is_valid_solana_address(address):
    try:
        # This will throw an exception if the address is not valid
        Pubkey(address)
        return True
    except:
        return False
def check_and_reserialize_solana_address(address):
    try:
        # Attempt to create a PublicKey instance
        pubkey = Pubkey(address)
        # Re-serialize the address (This step is typically not necessary for Solana addresses)
        reserialized_address = str(pubkey)
        return reserialized_address
    except Exception as e:
        print(f"Invalid Solana address: {e}")
        return address

def get_all_imports(keyword, case_sensitive=False, include_imports=True):
    directory = "/home/joben/Desktop/txns_sol/solana-py-master"
    keywords = make_list(keyword)
    all_imports = set()
    for root, _, files in os.walk(directory):
        for file in files:
            file_path = os.path.join(root, file)
            try:
                with open(file_path, 'r') as f:
                    for line in f:
                        line_processed = line if case_sensitive else line.lower()
                        if any((kw.lower() if not case_sensitive else kw) in line_processed for kw in keywords):
                            if include_imports and (line_processed.startswith('import ') or line_processed.startswith('from ')):
                                all_imports.add(line_processed.strip())
            except Exception as e:
                print(f"Error processing file {file_path}: {e}")

    for each in sorted(all_imports):
        print(each)
def get_account_params(base_public_key,new_public_key,fee,PublicKey,space=8):
        return sp.CreateAccountParams(
            from_pubkey=base_public_key,
            new_account_pubkey=new_public_key,
            lamports=fee,
            space=space,
            program_id=PublicKey(1),
        )

def get_keypair(key_pair_file):
    with open (key_pair_file) as kpf:
        keypair = kpf.read()
        keypair = keypair.replace('[', '').replace(']','')
        keypair = list(keypair.split(','))
        keypair = [int(i) for i in keypair]
    return Account(keypair[:32])

def get_account_info(client, public_key):
    account_info = client.get_account_info(public_key)
    print("--> Account Info: {}".format(account_info))
    balance = client.get_balance(public_key)
    print("--> Balance: {}".format(balance))
def get_ctx(rpc_url="https://api.devnet.solana.com",commitment="confirmed",timeout=30,blockhash_cache=True):
    cli=Client(rpc_url, commitment=Commitment(commitment), timeout=30,blockhash_cache=True)
    print("about to check connection")
    print("here is the connection: ", cli.is_connected())
    return cli
def pubkey_from_mint(mint):
    return Pubkey.from_string(mint)
def get_account_program_id_from_mint(mint,ctx=get_ctx()):
    pubkey = pubkey_from_mint(mint)
    return ctx.get_account_info_json_parsed(pubkey)

def get_owner(obj):
    return obj.value.owner
def get_program_id_from_mint(mint):
    pubkey = pubkey_from_mint(mint)
    program_id = get_program_id_from_pubkey(pubkey)
    return program_id

def get_account_program_id(pubkey,ctx=get_ctx()):
    return ctx.get_account_info_json_parsed(pubkey)

def get_program_id_from_pubkey(pubkey):
    accountProgramId=get_account_program_id(pubkey)
    return accountProgramId.value.owner

def get_accounts(payer,programid,ctx=get_ctx()):
    return ctx.get_token_accounts_by_owner_json_parsed(payer.pubkey()  ,TokenAccountOpts(program_id=programid)).value

def get_token_balance_lamports_from_account(account):
    return int(account.account.data.parsed['info']['tokenAmount']['amount'])

def get_mint_from_account(account):
    return account.account.data.parsed['info']['mint']

async def send_transaction(tx, signers, ctx):
    try:
        response = await ctx.send_transaction(tx, *signers)
        return response
    except Exception as e:
        print(f"Error sending transaction: {e}")
        return None

def get_recent_blockhash(ctx):
    return ctx.get_recent_blockhash()['result']['value']['blockhash']

def get_token_supply(mint_pubkey, ctx):
    return ctx.get_token_supply(mint_pubkey)['result']['value']['amount']

def get_token_account_balance(account_pubkey, ctx):
    return ctx.get_token_account_balance(account_pubkey)['result']['value']['amount']

async def get_associated_token_account_from_mint_keypair_file(mint,wallet_keypair):
    if os.path.isfile(wallet_keypair):
        wallet_keypair = load_keypair_from_file(wallet_keypair)

    # Convert the mint address to a Pubkey object
    mint_pubkey = pubkey_from_mint(mint)

    # Get the associated token address for the wallet and the mint
    associated_token_address = await get_associated_token_address(
        wallet_keypair.public_key,  # The wallet public key
        mint_pubkey                 # The mint public key
    )
    return associated_token_address
"""
Check balance of the token in your wallet
"""
def checkB(TOKEN_TO_SWAP_SELL, payer, ctx):
    tokenPk = pubkey_from_mint(TOKEN_TO_SWAP_SELL)
    accountProgramId = get_account_program_id(tokenPk)
    programid = get_program_id_from_pubkey(tokenPk)
    accounts = get_accounts(payer,programid,ctx=get_ctx())
    for account in accounts:
        tokenBalanceLamports = get_token_balance_lamports_from_account(account)
        mint = get_mint_from_account(account)
        if tokenBalanceLamports > 0 and mint == TOKEN_TO_SWAP_SELL:
            return True
    return False



import os
from solders.keypair import Keypair
from abstract_security import get_env_value
import base58
import sys
import json
def load_from_private_key(env_key='AMM_P'):
    env_value = get_env_value(key=env_key)
    if env_value:
        return Keypair.from_base58_string(env_value)

def load_keypair_from_file(filename):
    curr = os.path.join(sys.path[0], 'data',  filename)
    with open(curr, 'r') as file:
        secret = json.load(file)
        secret_key = bytes(secret)
        # print(base58.b58encode(secret_key))
        return Keypair.from_bytes(secret_key)
